; MP3 Player, Device Drivers, http://www.pjrc.com/tech/mp3
; Copyright (c) 2000, PJRC.COM, LLC

; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.

; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

; Contact: paul@pjrc.com


; drivers for MP3 player hardware.  These drivers are loaded into
; bank1 of the flash memory, and are intended to be called from a
; program running in bank0.

.equ    location, 0xA000        ;where this program will exist

.equ	sta013_config_location, 0xC000
.equ	xilinx_bitstream_location, 0xD000
.equ	xilinx_alt_bitstream_location, 0x7000

.equ	return_addr, 0x0FF0	;jump here to return to caller in bank0



.equ    cout, 0x0030            ;Send Acc to serial port
.equ    cin, 0x0032             ;Get Acc from serial port
.equ    phex, 0x0034            ;Print Hex value of Acc
.equ    phex16, 0x0036          ;Print Hex value of DPTR
.equ    pstr, 0x0038            ;Print string pointed to by DPTR,
.equ    upper, 0x0040           ;Convert Acc to uppercase
.equ    pint8u, 0x004D          ;print Acc at an integer, 0 to 255
.equ    pint8, 0x0050           ;print Acc at an integer, -128 to 127
.equ    pint16u, 0x0053         ;print DPTR as an integer, 0 to 65535
.equ    newline, 0x0048         ;print CR/LF (13 and 10)
.equ	rst_devices, 0x006A	;give a reset to the STA013, Xilinx, and Flash


.equ	scl_pin, 0x90
.equ	sda_pin, 0x91
.equ	hc165_load_pin, 0x93
.equ	unused_pin, 0x94
.equ	xilinx_cclk_pin, 0x95
.equ	hc165_clk_pin, 0x95
.equ	hc165_data_pin, 0x96
.equ	xilinx_din_pin, 0x97
.equ	flash_a16_pin, 0xB4
.equ	shutdown_pin, 0xB5



.org    0xA000
.db     0xA5,0xE5,0xE0,0xA5     ;signiture bytes
.db     0,255,0,0               ;id
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;user defined
.db     255,255,255,255         ;length and checksum (255=unused)
.db     "Hardware Drivers",0


.org	0xA040

;entry table... call to 0x0FE0 (within PM2) with one of these
;addresses in DPTR to call these drivers from bank0

	ljmp	xdownload	;0xA040  download xilinx bitstream
	ljmp	sta013_init	;0xA043  initialize the sta013
	ljmp	shutdown	;0xA046  shut off the power supply
	ljmp	alt_xdownload	;0xA049  download xilinx bitstream
	ljmp	sta013_rd	;0xA04C  read from the sta013
	ljmp	sta013_wr	;0xA04F  write to the sta013




	;download a bitstream to the xilinx chip.
	;return C=0 if ok, C=1 if an error

xdownload:
	mov	dptr, #xilinx_bitstream_location - 1
	acall	get_next_byte
	cjne	a, #0xFF, no_bitstream	;fill byte
	acall	get_next_byte
	cjne	a, #0x04, no_bitstream	;preamble, upper 4 bits of length
	acall	get_next_byte
	cjne	a, #0xE8, no_bitstream	;next 8 bits of length
	acall	get_next_byte
	cjne	a, #0xFA, no_bitstream	;next 8 bits of length
	acall	get_next_byte
	cjne	a, #0xF9, no_bitstream	;lower 4 bits length, fill bits

	;reset the xilinx chip... also resets the STA013 and flash
	lcall	rst_devices

	;wait for 3ms for the xilinx chip to be ready
	;at 7.3728 MHz / 12, that's 1843 instruction cycles

	djnz	r0, *		;512 cycles
	djnz	r0, *		;512 cycles
	djnz	r0, *		;512 cycles
	mov	r0, #153	;1 cycle
	djnz	r0, *		;306 cycles

	setb	xilinx_din_pin
	setb	xilinx_cclk_pin
	mov	dptr, #xilinx_bitstream_location
xd_loop1:
	clr	a
	movc	a, @a+dptr
	inc	dptr
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin

	mov	a, dpl
	cjne	a, #(xilinx_bitstream_location + 11968) & 255, xd_loop1
	mov	a, dph
	cjne	a, #(xilinx_bitstream_location + 11968) >> 8, xd_loop1

	setb	xilinx_din_pin
	clr	c
	ljmp	return_addr

get_next_byte:
	inc	dptr
	clr	a
	movc	a, @a+dptr
	ret

no_bitstream:
	setb	c
	ljmp	return_addr








	;download a bitstream to the xilinx chip.
	;return C=0 if ok, C=1 if an error

alt_xdownload:
	mov	dptr, #xilinx_alt_bitstream_location - 1
	acall	get_next_byte
	cjne	a, #0xFF, no_bitstream	;fill byte
	acall	get_next_byte
	cjne	a, #0x04, no_bitstream	;preamble, upper 4 bits of length
	acall	get_next_byte
	cjne	a, #0xE8, no_bitstream	;next 8 bits of length
	acall	get_next_byte
	cjne	a, #0xFA, no_bitstream	;next 8 bits of length
	acall	get_next_byte
	cjne	a, #0xF9, no_bitstream	;lower 4 bits length, fill bits

	;reset the xilinx chip... also resets the STA013 and flash
	lcall	rst_devices

	;wait for 3ms for the xilinx chip to be ready
	;at 7.3728 MHz / 12, that's 1843 instruction cycles

	djnz	r0, *		;512 cycles
	djnz	r0, *		;512 cycles
	djnz	r0, *		;512 cycles
	mov	r0, #153	;1 cycle
	djnz	r0, *		;306 cycles

	setb	xilinx_din_pin
	setb	xilinx_cclk_pin
	mov	dptr, #xilinx_alt_bitstream_location
alt_xd_loop1:
	clr	a
	movc	a, @a+dptr
	inc	dptr
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin
	rrc	a
	mov	xilinx_din_pin, c
	clr	xilinx_cclk_pin
	setb	xilinx_cclk_pin

	mov	a, dpl
	cjne	a, #(xilinx_alt_bitstream_location + 11968) & 255, alt_xd_loop1
	mov	a, dph
	cjne	a, #(xilinx_alt_bitstream_location + 11968) >> 8, alt_xd_loop1

	setb	xilinx_din_pin
	clr	c
	ljmp	return_addr






cout_dot:
	mov	a, #'.'
	ljmp	cout



sta013_init:

	acall	cout_dot

	;read the STA013 ident register
	mov	r4, #1
	acall	sta013_read
	jc	sta013_init1
	cjne	r3, #0xAC, sta013_init0
	sjmp	sta013_init2

sta013_init0:
	mov	a, r3
	lcall	phex
sta013_init1:
	mov	a, #'*'
	lcall	cout

	;read the STA013 ident register
	mov	r4, #1
	acall	sta013_read
	jc	sta013_error
	cjne	r3, #0xAC, sta013_error

sta013_init2:
	acall	cout_dot


	;check to see if the sta013 data file is loaded into memory

	mov	dptr, #sta013_config_location
	acall	check_data_file
	jc	sta013_error

	acall	cout_dot

	;now attempt to download the config data
	;to the STA013 chip

	acall	cout_dot

	acall	sta013_config

	acall	cout_dot

	mov	r4, #20
	mov	r3, #1
	acall	sta013_write		;mute on
	jc	sta013_error

	acall	cout_dot

	mov	r4, #114
	mov	r3, #1
	acall	sta013_write		;run command
	jc	sta013_error

	;acall	cout_dot

	;mov	r4, #19
	;mov	r3, #1
	;acall	sta013_write		;begin play
	;jc	sta013_error

	;acall	cout_dot

	;mov	r4, #20
	;mov	r3, #0
	;acall	sta013_write		;mute off
	;jc	sta013_error

	;acall	cout_dot

	clr	c
	ljmp	return_addr

sta013_error:
	setb	c
	ljmp	return_addr










sta013_config:
	mov	dptr, #sta013_config_location
	acall	get_16_bits	;r6/r7 has cksum addr
	mov	a, r6
	anl	a, #11111110b
	mov	r6, a
cfgloop:acall	get_8_bits	;get the address
	mov	r4, a
	acall	get_8_bits	;get the data byte
	mov	r3, a
	acall	sta013_write	;send this byte to its address
	mov	a, r4
	add	a, #250
	jnz	cfgloop2
	;delay at least 60 us if the write was a soft reboot cmd
	mov	r5, #200
	djnz	r5, *
cfgloop2:
	mov	a, r6
	cjne	a, dpl, cfgloop	;send all the bytes
	mov	a, r7
	cjne	a, dph, cfgloop
	ret



	;checks for a data file in memory @dptr.  The test
	;is simple... the first 2 bytes are the location of
	;the file's last two bytes, that are a simple 16 bit
	;checksum of all the data bytes

check_data_file:
	acall	get_16_bits	;r6/r7 has cksum addr
	clr	a
	mov	r0, a		;r0/r1 will hold the computed cksum
	mov	r1, a
ckloop:	acall	get_8_bits	;get each byte
	add	a, r0		;add it to the cksum
	mov	r0, a
	clr	a
	addc	a, r1
	mov	r1, a
	mov	a, r6
	cjne	a, dpl, ckloop	;check all the data bytes
	mov	a, r7
	cjne	a, dph, ckloop
	acall	get_8_bits	;get cksum lsb
	mov	b, r0
	cjne	a, b, ckfail
	acall	get_8_bits	;get cksum msb
	mov	b, r1
	cjne	a, b, ckfail
	clr	c
	ret
ckfail:	setb	c

	mov	a, #'!'
	lcall	cout
	ljmp	0

	ret



get_8_bits:
	clr	a
	movc	a, @a+dptr
	inc	dptr
	ret

get_16_bits:
	acall	get_8_bits
	mov	r6, a
	acall	get_8_bits
	mov	r7, a
	ret
	


	;allow the main program to call the read and write routines
sta013_rd:
	acall	sta013_read
	ljmp	return_addr
sta013_wr:
	acall	sta013_write
	ljmp	return_addr




	;read a byte from the sta013 at address r4 and return it
	;in r3.  Carry cleared if success, set for fail
sta013_read:
	acall	i2c_start
	mov	a, #10000110b
	acall	i2c_write		;wr device addr, wr cmd
	jc	s013r3
	mov	a, r4
	acall	i2c_write		;wr addr of data to read
	jc	s013r3
	acall	i2c_stop
	acall	i2c_start
	mov	a, #10000111b		;wr device addr, rd cmd
	acall	i2c_write
	jc	s013r3
	acall	i2c_read
	mov	r3, a
	clr	c
s013r3: ajmp	i2c_stop



	;write a byte in r3 to the sta013 at address r4
	;Carry cleared if success, set for fail
sta013_write:
	acall	i2c_start
	mov	a, #10000110b
	acall	i2c_write
	jc	s013w3
	mov	a, r4
	acall	i2c_write
	jc	s013w3
	mov	a, r3
	acall	i2c_write
s013w3: ajmp	i2c_stop	;i2c stop doesn't change carry bit







i2c_read:  ;reads i2c bus into acc, send ack bit from Carry
	rlc	a
        mov     r5, #8
        setb    sda_pin
	nop
i2cr:   nop
        setb    scl_pin
        nop
        nop
        mov     c, sda_pin
        rlc     a
	nop
        clr     scl_pin
        djnz    r5, i2cr
        mov     sda_pin, c
	nop
	nop
        setb    scl_pin
	nop
	nop
        clr     scl_pin
        ret


i2c_write:  ;writes acc to i2c bus, return ack bit in Carry
        mov     r5, #8
i2cw:   rlc     a
        mov     sda_pin, c
	nop
        setb    scl_pin
	nop
        clr     scl_pin
        djnz    r5, i2cw
        rlc     a
        setb    sda_pin
	nop
        setb    scl_pin
	nop
        mov     c, sda_pin
        clr     scl_pin
        ret


i2c_start:
        setb    sda_pin
        setb    scl_pin
	nop
	nop
        clr     sda_pin
	nop
	nop
        clr     scl_pin
        ret

i2c_stop:
        clr     sda_pin
	nop
	nop
        setb    scl_pin
	nop
	nop
        setb    sda_pin
        ret

dly:
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	ret
	


shutdown:
	mov	r0, #250
sd2:
	clr	shutdown_pin
	setb	shutdown_pin
	djnz	r0, sd2
	ljmp	return_addr






mesg_ok:.db     "ok", 13, 10, 0
mesg_nf:.db     "fail", 13, 10, 0


